// Quantize all selected events to frames. Rounds the time to the nearest frame boundry.
//
// To use:
// 1. Select the desired events
// 2. Run the script
//
// The script looks at each selected event on every track and quantizes the start and end times and optionally the fade
// times. 
// There are variables below that you can change to modify what quantization is applied to.
//
// Author: Randall Campbell, info@peachrock.com, www.peachrock.com
//  Copyright 2004-2005, Peach Rock Productions, LLC.
// You are free to use or modify this code as long as the copyright information is not removed.
// This software is provided AS IS, no warranty is expressed or implied

import System;
import System.Windows.Forms;
import System.Collections;
import ScriptPortal.Vegas;

// Set to true to only quantize video events
var videoOnly : boolean = false;

// Set to true to only quantize selected events
var selectedOnly : boolean = false;

// Set to true to also quantize fade times
var fades : boolean = true;

// Loop through all of the tracks and quantize the desired events
var count : int = 0;
for (var track in Vegas.Project.Tracks)
{
    if (!track.IsVideo() && videoOnly)
		continue;
	
	for (var trackEvent in track.Events)
	{
		if (!trackEvent.Selected && selectedOnly)
			continue;
			
		var start : Timecode = Quantize(trackEvent.Start);
		var end : Timecode = Quantize(trackEvent.Start + trackEvent.Length);
		trackEvent.AdjustStartLength(start, end - start, true);
		
		if ( track.IsVideo() && fades)
		{
			// Kill fades less than one frame for video events
			if (trackEvent.FadeIn.Length.Nanos > 0 && trackEvent.FadeIn.Length.FrameCount == 0)
				trackEvent.FadeIn.Length = Timecode.FromFrames(0);
			if (trackEvent.FadeOut.Length.Nanos > 0 && trackEvent.FadeOut.Length.FrameCount == 0)
				trackEvent.FadeOut.Length = Timecode.FromFrames(0);
				
			// Quantize the fades
			trackEvent.FadeIn.Length = Quantize(trackEvent.FadeIn.Length);
			trackEvent.FadeOut.Length = Quantize(trackEvent.FadeOut.Length);
		}
		
		count++;
	}
}
MessageBox.Show("Quantized " + count + " events to frame boundries.");

// Round the timecode value to the nearest frame
// Returns the quantized timecode value
function Quantize(timecode : Timecode) : Timecode
{
	var newTimecode : Timecode = new Timecode();
	newTimecode.Nanos = timecode.Nanos;
		
	var originalNanos : long = newTimecode.Nanos;
	newTimecode.FrameCount = newTimecode.FrameCount;
	var nano1 : long = newTimecode.Nanos;
	newTimecode.FrameCount = newTimecode.FrameCount + 1;
	var nano2 : long = newTimecode.Nanos;
	var nanoMiddle : long = Convert.ToInt64((nano2 - nano1) / 2);
	if (originalNanos < nano1 + nanoMiddle)
		newTimecode.FrameCount = newTimecode.FrameCount - 1;

	return newTimecode;
}
